
#    Copyright © 2010 Brandon L Black <blblack@gmail.com> and Jay Reitz <jreitz@gmail.com>
#
#    This file is part of gdnsd.
#
#    gdnsd is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    gdnsd is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with gdnsd.  If not, see <http://www.gnu.org/licenses/>.
#

# Dynamic CNAME plugin tests

use _GDT ();
use FindBin ();
use File::Spec ();
use Test::More tests => 11;

my $soa = 'example.com 86400 SOA ns1.example.com hostmaster.example.com 1 7200 1800 259200 900';

my $pid = _GDT->test_spawn_daemon(File::Spec->catfile($FindBin::Bin, 'gdnsd.conf'));

_GDT->test_dns(
    qname => 'example.com', qtype => 'SOA',
    answer => $soa,
);

# null dynamic CNAME plugin
_GDT->test_dns(
    qname => 'n.example.com', qtype => 'A',
    answer => 'n.example.com 86400 CNAME .',
);

# dynamic CNAME to A
_GDT->test_dns(
    qname => 'toa.example.com', qtype => 'A',
    answer => ['toa.example.com 600 CNAME a.example.com.',
                  'a.example.com 86400 A 8.8.4.4']
);

# null dynamic CNAME plugin
_GDT->test_dns(
    qname => 'n-x.example.com', qtype => 'A',
    answer => 'n-x.example.com 86400 CNAME .',
);

# dynamic CNAME to A (again)
_GDT->test_dns(
    qname => 'toa.example.com', qtype => 'A',
    answer => ['toa.example.com 600 CNAME a.example.com.',
               'a.example.com 86400 A 8.8.4.4']
);

# dynamic CNAME to missing label
_GDT->test_dns(
    qname => 'tomissing.example.com', qtype => 'A',
    header => { rcode => 'NXDOMAIN' },
    answer => 'tomissing.example.com 600 CNAME missing.example.com',
    auth => $soa,
    stats => [qw/udp_reqs nxdomain/],
);

# dynamic CNAME to external domain
_GDT->test_dns(
    qname => 'cdn.example.com', qtype => 'A',
    answer => 'cdn.example.com 600 CNAME cdn.net.',
);

# chain from static to dynamic cname and back to A record
_GDT->test_dns(
    qname => 'ctodyn.example.com', qtype => 'A',
    answer => ['ctodyn.example.com 86400 CNAME toa.example.com',
               'toa.example.com 600 CNAME a.example.com',
               'a.example.com 86400 A 8.8.4.4', ]
);

########
#  Below should pass once CNAME "chaining" for dynamic CNAMES has been added to dnspacket

_GDT->test_dns(
    qname => 'f42.example.com', qtype => 'A',
    answer => ['f42.example.com 600 CNAME n-x.example.com',
               'n-x.example.com 86400 CNAME .'],
);

_GDT->test_kill_daemon($pid);
